function [L,U,R] = ilu0_jki(A)
%
% ILU(0) factorization (without pivoting), jki version, didactic purpose
% column-contiguos data structure (Fortran, Matlab, Octave)
%
% A = L * U - R (Saad's notation)

% no extra storage needed
n = length (A);
R = spalloc (n, n, n ^ 2 - nnz (A));
for j = 1:n
  for k = 1:j - 1
    for i = k + 1:n
      if (A(i, j) ~= 0)
	A(i, j) = A(i, j) - A(i, k) * A(k, j); % j-th column is changed
      else
	R(i, j) = R(i, j) + A(i, k) * A(k, j);
      end
    end
  end
  for i = j + 1:n
    A(i, j) = A(i, j) / A(j, j); % j-th column is changed 
			         % (assuming A(j, j) ~= 0)
  end
end
% the matrix A now contains exactly U in the triu part and L (except the
% ones on the diagonal) on the tril part
U = triu (A);
L = A - U + speye (size (A));
%!test
%! A = sprand (10, 10, 3 / 10) + speye (10);
%! [L, U, R] = ilu0_jki(A);
%! assert(A, L * U - R, 1e-14)